import CSSClasses from '../Helpers/CSSClasses';
import Conditions from '../Helpers/Conditions';
import ValueCallback from '../Helpers/ValueCallback';
/**
 * Field class.
 *
 * @since 1.0.0
 */
export default class Field {
    id;
    type;
    label;
    placeholder;
    options;
    prefix;
    suffix;
    help;
    classes;
    labelClasses;
    inputClasses;
    size;
    conditions;
    validations;
    replacements;
    valueDefault;
    valueCallback;
    formValues;
    errors = [];
    isValidated = false;
    /**
     * Initializing field.
     *
     * @param fieldData  Field data.
     * @param formValues Form values.
     *
     * @since 1.0.0
     */
    constructor(fieldData, formValues) {
        this.formValues = formValues;
        this.id = fieldData.id;
        this.type = fieldData.type;
        this.label = fieldData.label;
        this.prefix = fieldData.prefix;
        this.suffix = fieldData.suffix;
        this.placeholder = fieldData.placeholder;
        this.options = fieldData.options;
        this.help = fieldData.help;
        this.valueDefault = fieldData.valueDefault;
        this.valueCallback = fieldData.valueCallback !== undefined ? new ValueCallback(fieldData.valueCallback, formValues) : undefined;
        this.validations = fieldData.validations;
        this.replacements = fieldData.replacements;
        this.conditions = fieldData.conditions;
        this.labelClasses = new CSSClasses(fieldData.labelClasses);
        this.inputClasses = new CSSClasses(fieldData.inputClasses);
        this.classes = new CSSClasses(fieldData.classes);
        this.size = fieldData.size;
    }
    /**
     * Updating field assets
     *
     * @since 1.0.0
     */
    updateAssets() {
        if (!this.isValidated) {
            return;
        }
        if (this.getErrors().length > 0) {
            this.classes.remove('is-valid');
            this.inputClasses.remove('is-valid');
            this.classes.add('is-invalid');
            this.inputClasses.add('is-invalid');
        }
        else {
            this.classes.remove('is-invalid');
            this.inputClasses.remove('is-invalid');
            this.classes.add('is-valid');
            this.inputClasses.add('is-valid');
        }
    }
    /**
     * Replace a value by field replacements.
     *
     * @param value Value to be replaced.
     *
     * @returns Replaced value.
     *
     * @since 1.0.0
     */
    replace(value) {
        if (this.replacements === undefined) {
            return value;
        }
        this.replacements.forEach(replacement => {
            value = value.replace(replacement.from, replacement.to);
        });
        return value;
    }
    /**
     * Checks if conditions for displaying field are met.
     *
     * @returns True if conditions are met, false otherwise.
     *
     * @since 1.0.0
     */
    conditionsFulfilled() {
        if (this.conditions === undefined || this.conditions.length === 0) {
            return true;
        }
        let conditions = new Conditions(this.conditions, this.formValues);
        return conditions.fullfilled();
    }
    /**
     * Get errors of the field.
     *
     * @returns Erros of the field.
     *
     * @since 1.0.0
     */
    getErrors() {
        return this.errors;
    }
    /**
     * Is field visible.
     *
     * @returns True if field is visible, false otherwise.
     *
     * @since 1.0.0
     */
    isVisible() {
        return this.conditionsFulfilled();
    }
}
